@@ -1,13 +1,11 @@
 """
-This module implements a plotter for the Fermi-Surface of a material
-todo:
-* Remap into Brillioun zone (Wigner Seitz cell)
-* Get Latex working for labels
-* Do projections onto arbitrary surface
-* Comment more
-* Think about classes/methods, maybe restructure depending on sumo layout
+This module implements plotters for Fermi surfaces and Fermi slices.
+
+TODO:
+- Projections onto arbitrary surface
 """
 
+from pathlib import Path
 from typing import Any, Dict, List, Optional, Tuple, Union
 
 import numpy as np
@@ -21,17 +19,16 @@ from trimesh import transform_points
 from ifermi.brillouin_zone import ReciprocalCell, ReciprocalSlice
 from ifermi.fermi_surface import FermiSlice, FermiSurface
 
-try:
-    import plotly
-    import plotly.express as px
-except ImportError:
-    plotly = False
-
 try:
     import mayavi.mlab as mlab
 except ImportError:
     mlab = False
 
+try:
+    import kaleido
+except ImportError:
+    kaleido = False
+
 try:
     from crystal_toolkit.core.scene import Lines, Scene, Spheres, Surface
 
@@ -85,11 +82,13 @@ _mayavi_rs_style = {
     "tube_radius": 0.005,
     "representation": "surface",
 }
+_default_azimuth = 45.0
+_default_elevation = 35.0
 
 
 class FermiSurfacePlotter(MSONable):
     """
-    Class to plot FermiSurface.
+    Class to plot a FermiSurface.
     """
 
     def __init__(self, fermi_surface: FermiSurface):
@@ -104,7 +103,8 @@ class FermiSurfacePlotter(MSONable):
 
     @staticmethod
     def get_symmetry_points(
-        fermi_surface: FermiSurface, symprec: float = 1e-3,
+        fermi_surface: FermiSurface,
+        symprec: float = 1e-3,
     ) -> Tuple[np.ndarray, List[str]]:
         """
         Get the high symmetry k-points and labels for the Fermi surface.
@@ -126,45 +126,48 @@ class FermiSurfacePlotter(MSONable):
 
         return kpoints, labels
 
-    def plot(
+    def get_plot(
         self,
         plot_type: str = "plotly",
-        interactive: bool = True,
-        filename: str = "fermi_surface.png",
         spin: Optional[Spin] = None,
         colors: Optional[Union[str, dict, list]] = None,
+        azimuth: float = _default_azimuth,
+        elevation: float = _default_elevation,
         **plot_kwargs,
     ):
         """
-        Plot the Fermi surface and save the image to a file.
+        Plot the Fermi surface.
 
         Args:
             plot_type: Method used for plotting. Valid options are: "matplotlib",
                 "plotly", "mayavi", "crystal_toolkit".
-            interactive: Whether to enable interactive plots.
-            filename: Output filename.
             spin: Which spin channel to plot. By default plot both spin channels if
                 available.
             colors: See the docstring for ``get_isosurfaces_and_colors()`` for the
                 available options.
+            azimuth: The azimuth of the viewpoint in degrees. i.e. the angle subtended
+                by the position vector on a sphere projected on to the x-y plane.
+            elevation: The zenith angle of the viewpoint in degrees, i.e. the angle
+                subtended by the position vector and the z-axis.
             **plot_kwargs: Other keyword arguments supported by the individual plotting
                 methods.
         """
         plot_kwargs.update(
-            dict(filename=filename, interactive=interactive, colors=colors, spin=spin)
+            {"colors": colors, "spin": spin, "azimuth": azimuth, "elevation": elevation}
         )
-        if plot_type == "mpl":
-            self.plot_matplotlib(**plot_kwargs)
+        if plot_type == "matplotlib":
+            plot = self.get_matplotlib_plot(**plot_kwargs)
         elif plot_type == "plotly":
-            self.plot_plotly(**plot_kwargs)
+            plot = self.get_plotly_plot(**plot_kwargs)
         elif plot_type == "mayavi":
-            self.plot_mayavi(**plot_kwargs)
-        elif plot_type == "mayavi":
-            self.plot_crystal_toolkit(**plot_kwargs)
+            plot = self.get_mayavi_plot(**plot_kwargs)
+        elif plot_type == "crystal_toolkit":
+            plot = self.get_crystal_toolkit_plot(**plot_kwargs)
         else:
-            types = ["mpl", "plotly", "mayavi", "crystal_toolkit"]
+            types = ["matplotlib", "plotly", "mayavi", "crystal_toolkit"]
             error_msg = "Plot type not recognised, valid options: {}".format(types)
             raise ValueError(error_msg)
+        return plot
 
     def get_isosurfaces_and_colors(
         self,
@@ -181,6 +184,7 @@ class FermiSurfacePlotter(MSONable):
             spin: Which spin channel to select. By default will return the isosurfaces
                 for both spin channels if available.
             colors: The color specification. Valid options are:
+
                 - A list of colors.
                 - A dictionary of ``{Spin.up: color1, Spin.down: color2}``.
                 - A string specifying which matplotlib colormap to use. See
@@ -202,42 +206,44 @@ class FermiSurfacePlotter(MSONable):
 
         if colors is None and plot_type == "plotly":
             colors = _get_plotly_colors(self.fermi_surface.isosurfaces, spin)
-
         else:
             colors = _get_random_colors(colors, self.fermi_surface.isosurfaces, spin)
 
         return isosurfaces, colors
 
-    def plot_matplotlib(
+    def get_matplotlib_plot(
         self,
-        interactive: bool = True,
         bz_linewidth: float = 0.9,
         spin: Optional[Spin] = None,
         colors: Optional[Union[str, dict, list]] = None,
-        title: str = None,
-        filename: str = "fermi_surface.png",
+        azimuth: float = _default_azimuth,
+        elevation: float = _default_elevation,
     ):
         """
         Plot the Fermi surface using matplotlib.
 
         Args:
-            interactive: Whether to enable interactive plots.
             bz_linewidth: Brillouin zone line width.
             spin: Which spin channel to plot. By default plot both spin channels if
                 available.
             colors: See the docstring for ``get_isosurfaces_and_colors()`` for the
                 available options.
-            title: The title of the plot.
-            filename: The output file name.
+            azimuth: The azimuth of the viewpoint in degrees. i.e. the angle subtended
+                by the position vector on a sphere projected on to the x-y plane.
+            elevation: The zenith angle of the viewpoint in degrees, i.e. the angle
+                subtended by the position vector and the z-axis.
+
+        Returns:
+            matplotlib pyplot object.
         """
         import matplotlib.pyplot as plt
         from mpl_toolkits.mplot3d.art3d import Line3DCollection
 
-        fig = plt.figure(figsize=(10, 10))
-        ax = fig.add_subplot(111, projection="3d")
+        fig = plt.figure(figsize=(6, 6))
+        ax = fig.add_subplot(111, projection="3d", proj_type="persp")
 
         isosurfaces, colors = self.get_isosurfaces_and_colors(
-            spin=spin, colors=colors, plot_type="mpl"
+            spin=spin, colors=colors, plot_type="matplotlib"
         )
 
         # create a mesh for each electron band which has an isosurfaces at the Fermi
@@ -254,44 +260,42 @@ class FermiSurfacePlotter(MSONable):
         ax.add_collection3d(lines)
 
         for coords, label in zip(*self._symmetry_pts):
-            ax.scatter(*coords, s=10, c="k")
-            ax.text(*coords, label, size=15, zorder=1)
-
-        if title is not None:
-            plt.title(title)
+            ax.scatter(*coords, s=20, c="k", zorder=20)
+            ax.text(*coords, "${}$".format(label), size=18, zorder=20)
 
         xlim, ylim, zlim = np.linalg.norm(self.rlat, axis=1) / 2
         ax.set(xlim=(-xlim, xlim), ylim=(-ylim, ylim), zlim=(-zlim, zlim))
-
+        ax.view_init(elev=elevation, azim=azimuth)
         ax.axis("off")
         plt.tight_layout()
 
-        if interactive:
-            plt.show()
-        else:
-            plt.savefig(filename, dpi=300)
+        return plt
 
-    @requires(plotly, "plotly option requires plotly to be installed.")
-    def plot_plotly(
+    def get_plotly_plot(
         self,
-        interactive: bool = True,
         spin: Optional[Spin] = None,
         colors: Optional[Union[str, dict, list]] = None,
-        filename: str = "fermi_surface.png",
+        azimuth: float = _default_azimuth,
+        elevation: float = _default_elevation,
     ):
         """
         Plot the Fermi surface using plotly.
 
         Args:
-            interactive: Whether to enable interactive plots.
             spin: Which spin channel to plot. By default plot both spin channels if
                 available.
             colors: See the docstring for ``get_isosurfaces_and_colors()`` for the
                 available options.
-            filename: The output file name.
+            azimuth: The azimuth of the viewpoint in degrees. i.e. the angle subtended
+                by the position vector on a sphere projected on to the x-y plane.
+            elevation: The zenith angle of the viewpoint in degrees, i.e. the angle
+                subtended by the position vector and the z-axis.
+
+        Returns:
+            Plotly figure object.
         """
         import plotly.graph_objs as go
-        from plotly.offline import init_notebook_mode, plot
+        from plotly.offline import init_notebook_mode
 
         init_notebook_mode(connected=True)
         isosurfaces, colors = self.get_isosurfaces_and_colors(spin=spin, colors=colors)
@@ -318,7 +322,6 @@ class FermiSurfacePlotter(MSONable):
             meshes.append(trace)
 
         # plot high symmetry labels
-        # labels = [i.replace(r"\Gamma", "\u0393") for i in self._symmetry_pts[1]]
         labels = ["${}$".format(i) for i in self._symmetry_pts[1]]
         x, y, z = self._symmetry_pts[0].T
         marker_style = dict(size=5, color="black")
@@ -334,35 +337,36 @@ class FermiSurfacePlotter(MSONable):
 
         # Specify plot parameters
         layout = go.Layout(
-            scene=scene,
-            showlegend=False,
-            title=go.layout.Title(text="", xref="paper", x=0),
+            scene=scene, showlegend=False, margin=go.layout.Margin(l=0, r=0, b=0, t=0)
         )
         fig = go.Figure(data=meshes, layout=layout)
+        fig.update_layout(scene_camera=_get_plotly_camera(azimuth, elevation))
 
-        if interactive:
-            plot(fig, include_mathjax="cdn")
-        else:
-            plotly.io.write_image(fig, str(filename), width=600, height=600, scale=5)
+        return fig
 
     @requires(mlab, "mayavi option requires mayavi to be installed.")
-    def plot_mayavi(
+    def get_mayavi_plot(
         self,
-        interactive: bool = True,
         spin: Optional[Spin] = None,
         colors: Optional[Union[str, dict, list]] = None,
-        filename: str = "fermi_surface.png",
+        azimuth: float = _default_azimuth,
+        elevation: float = _default_elevation,
     ):
         """
         Plot the Fermi surface using mayavi.
 
         Args:
-            interactive: Whether to enable interactive plots.
             spin: Which spin channel to plot. By default plot both spin channels if
                 available.
             colors: See the docstring for ``get_isosurfaces_and_colors()`` for the
                 available options.
-            filename: The output file name.
+            azimuth: The azimuth of the viewpoint in degrees. i.e. the angle subtended
+                by the position vector on a sphere projected on to the x-y plane.
+            elevation: The zenith angle of the viewpoint in degrees, i.e. the angle
+                subtended by the position vector and the z-axis.
+
+        Returns:
+            mlab figure object.
         """
         from mlabtex import mlabtex
 
@@ -384,25 +388,21 @@ class FermiSurfacePlotter(MSONable):
         for coords, label in zip(self._symmetry_pts[0], labels):
             mlabtex(*coords, label, **_mayavi_high_sym_label_style)
 
-        if isinstance(self.reciprocal_space, ReciprocalCell):
-            mlab.view(azimuth=0, elevation=60, distance=8)
-        else:
-            mlab.view(azimuth=235, elevation=60, distance=8)
+        mlab.view(azimuth=azimuth - 180, elevation=elevation - 90, distance="auto")
 
-        if interactive:
-            mlab.show()
-        else:
-            mlab.savefig(str(filename), figure=mlab.gcf())
+        return mlab
 
     @requires(
         crystal_toolkit,
         "crystal_toolkit option requires crystal_toolkit to be installed.",
     )
-    def plot_crystal_toolkit(
+    def get_crystal_toolkit_plot(
         self,
         spin: Optional[Spin] = None,
         colors: Optional[Union[str, dict, list]] = None,
         opacity: float = 1.0,
+        azimuth: float = _default_azimuth,
+        elevation: float = _default_elevation,
     ) -> "Scene":
         """
         Get a crystal toolkit Scene showing the Fermi surface. The Scene can be
@@ -417,6 +417,13 @@ class FermiSurfacePlotter(MSONable):
                 available options.
             opacity: Opacity of surface. Note that due to limitations of WebGL,
                 overlapping semi-transparent surfaces might result in visual artefacts.
+            azimuth: The azimuth of the viewpoint in degrees. i.e. the angle subtended
+                by the position vector on a sphere projected on to the x-y plane.
+            elevation: The zenith angle of the viewpoint in degrees, i.e. the angle
+                subtended by the position vector and the z-axis.
+
+        Returns:
+            Crystal-toolkit scene.
         """
 
         # The implementation here is very similar to the plotly implementation, except
@@ -511,33 +518,33 @@ class FermiSlicePlotter(object):
 
         return kpoints[:, :2], labels
 
-    def plot(
+    def get_plot(
         self,
-        filename: str = "fermi_slice.png",
         spin: Optional[Spin] = None,
         colors: Optional[Union[str, dict, list]] = "viridis",
-        show: bool = False,
     ):
         """
-        Plot the Fermi surface and save the image to a file.
+        Plot the Fermi slice.
 
         Args:
-            filename: Output filename.
             spin: Which spin channel to plot. By default plot both spin channels if
                 available.
             colors: The color specification. Valid options are:
+
                 - A list of colors.
                 - A dictionary of ``{Spin.up: color1, Spin.down: color2}``.
                 - A string specifying which matplotlib colormap to use. See
                   https://matplotlib.org/tutorials/colors/colormaps.html for more
                   information.
                 - ``None``, in which case the colors will be chosen randomly.
-            show: Show the plot before saving to file.
+
+        Returns:
+            matplotlib pyplot object.
         """
         import matplotlib.pyplot as plt
         from matplotlib.collections import LineCollection
 
-        fig = plt.figure(figsize=(6, 6))
+        fig = plt.figure(figsize=(5, 5))
         ax = fig.add_subplot(111)
 
         # get a rotation matrix that will align the longest slice length along the
@@ -560,17 +567,15 @@ class FermiSlicePlotter(object):
 
         for coords, label in zip(*self._symmetry_pts):
             coords = np.dot(coords, rotation)
-            ax.scatter(*coords, s=10, c="k")
+            ax.scatter(*coords, s=20, c="k")
             label = label.replace(r"\Gamma", r"$\Gamma$")
-            ax.text(*coords, " " + label, size=15, zorder=1)
+            ax.text(*coords, " " + label, size=18, zorder=10)
 
         ax.autoscale(enable=True)
         ax.axis("equal")
         ax.axis("off")
 
-        if show:
-            plt.show()
-        plt.savefig(filename, dpi=300, bbox_inches="tight")
+        return plt
 
     def get_slices_and_colors(
         self,
@@ -584,6 +589,7 @@ class FermiSlicePlotter(object):
             spin: Which spin channel to select. By default will return the slices
                 for both spin channels if available.
             colors: The color specification. Valid options are:
+
                 - A list of colors.
                 - A dictionary of ``{Spin.up: color1, Spin.down: color2}``.
                 - A string specifying which matplotlib colormap to use. See
@@ -608,7 +614,95 @@ class FermiSlicePlotter(object):
         return slices, colors
 
 
-def kpoints_to_first_bz(kpoints: np.ndarray, tol=1e-5) -> np.ndarray:
+def show_plot(plot: Any):
+    """Display a plot.
+
+    Args:
+        plot: A plot object from ``FermiSurfacePlotter.get_plot()``. Supports matplotlib
+            pyplot objects, plotly figure objects, and mlab figure objects.
+    """
+    plot_type = get_plot_type(plot)
+
+    if plot_type == "matplotlib":
+        plot.show()
+    elif plot_type == "plotly":
+        from plotly.offline import plot as show_plotly
+
+        show_plotly(plot, include_mathjax="cdn", filename="fermi-surface.html")
+    elif plot_type == "mayavi":
+        plot.show()
+
+
+def save_plot(
+    plot: Any,
+    filename: Union[Path, str],
+    scale: float = 4,
+):
+    """Save a plot to file.
+
+    Args:
+        plot: A plot object from ``FermiSurfacePlotter.get_plot()``. Supports matplotlib
+            pyplot objects, plotly figure objects, and mlab figure objects.
+        filename: The output filename.
+        scale: Scale for the figure size. Increases resolution but does not change the
+            relative size of the figure and text.
+    """
+    plot_type = get_plot_type(plot)
+    filename = str(filename)
+    if plot_type == "matplotlib":
+        # default dpi is ~100
+        plot.savefig(filename, dpi=scale * 100, bbox_inches="tight")
+    elif plot_type == "plotly":
+        if kaleido is None:
+            raise ValueError(
+                "kaleido package required to save static ploty images\n"
+                "please install it using:\npip install kaleido"
+            )
+        plot.write_image(
+            filename,
+            engine="kaleido",
+            scale=scale,
+        )
+    elif plot_type == "mayavi":
+        plot.savefig(filename, magnification=scale)
+
+
+def _get_plotly_camera(azimuth: float, elevation: float) -> Dict[str, Dict[str, float]]:
+    """Get plotly viewpoint from azimuth and elevation."""
+    azimuth = np.radians(azimuth)
+    elevation = np.radians(elevation)
+    norm = np.linalg.norm([1.25, 1.25, 1.25])  # default plotly vector distance
+    x = np.sin(azimuth) * np.cos(elevation) * norm
+    y = np.cos(azimuth) * np.cos(elevation) * norm
+    z = np.sin(elevation) * norm
+    return dict(
+        up=dict(x=0, y=0, z=1), center=dict(x=0, y=0, z=0), eye=dict(x=x, y=y, z=z)
+    )
+
+
+def get_plot_type(plot: Any) -> str:
+    """Gets the plot type.
+
+    Args:
+        plot: A plot object from ``FermiSurfacePlotter.get_plot()``. Supports matplotlib
+            pyplot objects, plotly figure objects, and mlab figure objects.
+
+    Returns:
+        The plot type. Current options are "matplotlib", "mayavi", and "plotly".
+    """
+    from plotly.graph_objs import Figure
+
+    if isinstance(plot, Figure):
+        return "plotly"
+    elif hasattr(plot, "__name__"):
+        if "matplotlib" in plot.__name__:
+            return "matplotlib"
+        elif "mayavi" in plot.__name__:
+            return "mayavi"
+    raise ValueError("Unrecognised plot type.")
+
+
+def kpoints_to_first_bz(kpoints: np.ndarray, tol: float = 1e-5) -> np.ndarray:
     """Translate fractional k-points to the first Brillouin zone.
 
     I.e. all k-points will have fractional coordinates:
@@ -616,6 +710,7 @@ def kpoints_to_first_bz(kpoints: np.ndarray, tol=1e-5) -> np.ndarray:
 
     Args:
         kpoints: The k-points in fractional coordinates.
+        tol: Numerical tolerance.
 
     Returns:
         The translated k-points.
@@ -664,7 +759,10 @@ def _get_random_colors(
     return colors
 
 
-def _get_plotly_colors(objects: Dict[Spin, List[Any]], spins: List[Spin],) -> Any:
+def _get_plotly_colors(
+    objects: Dict[Spin, List[Any]],
+    spins: List[Spin],
+) -> Any:
     import plotly.express as px
 
     n_objects = sum([len(objects[spin]) for spin in spins])
